#Import the data "prdnadj10.csv" available in this folder
dd.H1 <- read.csv("prdnadj10.csv", header = T)
assign("mat.limits", dd.H1)
library(BLiP)

#Given fixed margins and odds ratio, the following function gets the cell probability
#for details please see https://stats.stackexchange.com/questions/241384/how-to-derive-2x2-cell-counts-from-contingency-table-margins-and-the-odds-ratio
#for a 2x2 table with a+b+c+d=1, a+b=beta, a+c=gamma and a*d=rho*b*c, get.p11 gets a=p11

get.p11 <- function(beta, gamma, rho, eps=1e-15) {
  a <- rho-1
  b <- (beta+gamma)*(1-rho)-1
  cc <- rho*beta*gamma
  
  if (abs(a) < eps) {
    z <- -cc / b
  } else {
    d <- b^2 - 4*a*cc
    if (d < eps*eps) s <- 0 else s <- c(-1,1)
    z <- (-b + s*sqrt(max(0, d))) / (2*a)
  }

  return(z)
}


#For a given marginal sums p1, p2 and odds ratio rho computes the coverage
#n.p1 and n.p2 for the number of partition points in p1 and p2 spaces
#lower and upper are the lower and upper confidence limits
make.paired.cov <- function(mat.limits, n.p1,n.p2,lower,upper,rho)
  {
    # mat.limits = (n1+1)(n2+10) by 2 matrix of limits
    # n.p1 = num of equi-spaced p1 pts at which coverage is to be 
    #         computed
    # n.p2 = num of equi-spaced p2 pts at which coverage is to be 
    #         computed
    n1 <- max(mat.limits[,1])
    n2 <- max(mat.limits[,2])
    n <- max(mat.limits[,1])
    
    num.p.vecs <- n.p1*n.p2
    
    p1 <- seq(from = 0.01 , to =.99, length = n.p1)
    p2 <- seq(from = 0.01, to =0.99, length = n.p2)
    cover <- matrix(NA,nrow=n.p1,ncol=n.p2)
    
    for(i in 1:n.p1) {
      for(j in 1:n.p2){
        delta <- p1[i]-p2[j]
        in.it <- ifelse((mat.limits[, lower] <= delta) & (delta <= mat.limits[, upper]),1,0)
        in.it.row <- in.it*(1:nrow(mat.limits))
        
        a.1<-mat.limits[, 2]
        b.1<-mat.limits[, 3]
        c.1<-mat.limits[, 4]
        a=0
        a<-get.p11(p1[i],p2[j],rho)
        pi.11=a

        if (pi.11<=p1[i] && pi.11<=p2[j])
        {
          p1.prob <- gamma(n+1)/(gamma(a.1+1)*gamma(c.1+1)*gamma(b.1+1)*gamma(n-a.1-b.1-c.1+1))*((pi.11^a.1) * ((p1[i]-pi.11)^b.1) *((p2[j]-pi.11)^c.1) *((1-p1[i]-p2[j]+pi.11)^(n-a.1-b.1-c.1)))
          tt=p1.prob*in.it
          ttmp<-c(p1.prob,in.it)
          t2<-sum(p1.prob)
          t1<-sum(tt)
          cover[i,j] <- sum(t1)
        }
      }
    }
    tmp <- list(p1=p1,p2=p2,coverage=cover)
    tmp
  }

p1p2.cov.explen<-function(n1)
{
  
  assign(paste("am",n1,90,sep="."),make.paired.cov(mat.limits,100,100,19,20,1))
  assign(paste("mvrwi",n1,90,sep="."),make.paired.cov(mat.limits,100,100,13,14,1))
  assign(paste("mvrac",n1,90,sep="."),make.paired.cov(mat.limits,100,100,15,16,1))
  assign(paste("mvrjf",n1,90,sep="."),make.paired.cov(mat.limits,100,100,17,18,1))
  assign(paste("tg",n1,90,sep="."),make.paired.cov(mat.limits,100,100,25,26,1))
  
  assign(paste("pf",n1,90,sep="."),make.paired.cov(mat.limits,100,100,9,10,1))
  assign(paste("cop",n1,90,sep="."),make.paired.cov(mat.limits,100,100,21,22,1))
  assign(paste("ex",n1,90,sep="."),make.paired.cov(mat.limits,100,100,27,28,1))
  
  tmp1<-list(get(paste("am",n1,"90",sep="."))$coverage,
              get(paste("mvrwi",n1,"90",sep="."))$coverage,
              get(paste("mvrac",n1,"90",sep="."))$coverage,
              get(paste("mvrjf",n1,"90",sep="."))$coverage,
              get(paste("tg",n1,"90",sep="."))$coverage,
              
              get(paste("pf",n1,"90",sep="."))$coverage,
              get(paste("cop",n1,"90",sep="."))$coverage,
              get(paste("ex",n1,"90",sep="."))$coverage	)
  
  assign("tmp1", tmp1)
  
  names(tmp1)<- c("AM","MVR-WI","MVR-AC","MVR-JF","TG","PF", "COP", "EX")
  tmp1
}  

tt<-p1p2.cov.explen(10) 
n1<-10
#Plotting coverage using BLip plot
par(las=1)  
blip(tt,
       box.p=(0:10)/10,
       graph.width="v",
       nclass=10,
       box.label.on=F,
       point.pch=183,
       point.cex=.6,ylab=names(tt),
       xlab="Coverage")
  abline(v=0.95)
  title(paste("Coverages of 95% Nominal Intervals for n =", n1,sep=""))
  #rm(tmp1)
  
  
  
  #--------------------------Expected Length Calculation-------------------------------
  #For a given marginal sums p1, p2 and odds ratio rho computes the coverage
  #n.p1 and n.p2 for the number of partition points in p1 and p2 spaces
  #lower and upper are the lower and upper confidence limits

  make.n1byn2.delta.exp.length<-function(mat.limits, n.p1,n.p2,lower,upper,rho)
  {
    n1 <- max(mat.limits[,1])
    n2 <- max(mat.limits[,2])
    n <- max(mat.limits[,1])
    num.p.vecs <- n.p1*n.p2
    rho=1
    p1 <- seq(from = .01, to =.99, length = n.p1)
    p2 <- seq(from = .01, to =.99, length = n.p2)
    exp.len <- matrix(0.,nrow= n.p1,ncol=n.p2)
    for(i in 1:n.p1) {
      for(j in 1:n.p2){
        delta <- p1[i]-p2[j]
        in.it <- (mat.limits[, lower]<= delta) & (delta <= mat.limits[, upper])
        in.it.row <- in.it*(1:nrow(mat.limits))
        
        a.1<-mat.limits[, 2]*in.it
        
        b.1<-mat.limits[, 3]*in.it
        
        c.1<-mat.limits[, 4]*in.it
        a<-get.p11(p1[i],p2[j],rho)
        
        pi.11=a
        
        #Probability computation using multinomial distribution 
        p1.prob <- gamma(n+1)/(gamma(a.1+1)*gamma(c.1+1)*gamma(b.1+1)*gamma(n-a.1-b.1-c.1+1))*((pi.11^a.1) * ((p1[i]-pi.11)^b.1) *((p2[j]-pi.11)^c.1) *((1-p1[i]-p2[j]+pi.11)^(n-a.1-b.1-c.1)))
        
        int.len<-mat.limits[,upper]-mat.limits[,lower]
        exp.len[i,j] <- sum(p1.prob*int.len*in.it)
      }
      
    }
    tmp <- list(p1=p1,p2=p2,exp.length=exp.len)
    #tmp
  }
 
  expected.length <-function(n1)
  {
    assign(paste("as05",n1,90,"expl",sep="."),make.n1byn2.delta.exp.length(mat.limits,100,100,19,20,1))
    assign(paste("as1",n1,90,"expl",sep="."),make.n1byn2.delta.exp.length(mat.limits,100,100,13,14,1))
    assign(paste("as2",n1,90,"expl",sep="."),make.n1byn2.delta.exp.length(mat.limits,100,100,15,16,1))
    assign(paste("ac",n1,90,"expl",sep="."),make.n1byn2.delta.exp.length(mat.limits,100,100,17,18,1))
    assign(paste("wi",n1,90,"expl",sep="."),make.n1byn2.delta.exp.length(mat.limits,100,100,25,26,1))
    
    assign(paste("pf",n1,90,"expl",sep="."),make.n1byn2.delta.exp.length(mat.limits,100,100,9,10,1))
    assign(paste("cop",n1,90,"expl",sep="."),make.n1byn2.delta.exp.length(mat.limits,100,100,21,22,1))
    assign(paste("ex",n1,90,"expl",sep="."),make.n1byn2.delta.exp.length(mat.limits,100,100,27,28,1))
    
    
    tmp2 <- list(get(paste("as05",n1,90,"expl",sep="."))$exp.length,
                get(paste("as1",n1,90,"expl",sep="."))$exp.length,
                get(paste("as2",n1,90,"expl",sep="."))$exp.length,
                get(paste("ac",n1,90,"expl",sep="."))$exp.length,
                get(paste("wi",n1,90,"expl",sep="."))$exp.length,
                
                get(paste("pf",n1,90,"expl",sep="."))$exp.length,
                get(paste("cop",n1,90,"expl",sep="."))$exp.length,
                get(paste("ex",n1,90,"expl",sep="."))$exp.length)
    
    
    names(tmp2)<- c("AM","MVR-WI","MVR-AC","MVR-JF","TG","PF", "COP", "EX")
    par(las=1)
    blip(tmp2,
         box.p=(0:10)/10,
         graph.width="v",
         nclass=10,
         box.label.on=F,
         point.pch=183,
         point.cex=.6,ylab=names(tmp2),
         xlab="Expected Length")
    title(paste("Expected lengths of 95% Nominal Intervals for n =", n1,sep=""))
    #rm(tmp2)
  }
  
  #Plotting expected lengths using BLip plot
  expected.length(10)
